﻿'Birthday manager by Philipp Erler
'Written in 2011
'Last change 31 | 01 | 2011
'You may change and distribute this code as you want
'You may not sell this program, this code or parts of it
'########################################################
'Manage birthdays with this tool
'Get a message if someone's birthday is today 
'########################################################

Imports Microsoft.Win32.Registry
Imports System.Security.Permissions

Public Class frmBirthdayMessenger

    Structure BirthdayInfo
        'content
        Public name As String
        Public preName As String
        Public birthDay As Date
        Public info As String

        'constructor
        Public Sub New(ByVal name As String, ByVal preName As String, ByVal birthday As Date, ByVal info As String)
            Me.name = name
            Me.preName = preName
            Me.birthDay = birthday
            Me.info = info
        End Sub
    End Structure

    Public birthdays As New List(Of BirthdayInfo)                                   'list of birthdays
    Private indexList As New List(Of Integer)                                       'list of birthday indices

    'save old date for update
    Private oldDate As Date = New Date(0)

    'ignore changes (used to show birthday info)
    Private ignoreChanges As Boolean = False

    'exit program
    Private Sub BeendenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
        Handles BeendenToolStripMenuItem.Click
        End
    End Sub

    'set check mark if autostart entry is set
    Private Sub OptionenToolStripMenuItem_DropDownOpening(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OptionenToolStripMenuItem.DropDownOpening

        'if already in autostart set checked
        Dim oRegKey As Microsoft.Win32.RegistryKey
        '64 vs 32 bit
        If Environment.Is64BitOperatingSystem Then
            oRegKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Run", False)
        Else
            oRegKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("SOFTWARE\Microsoft\Windows\CurrentVersion\Run", False)
        End If

        'read entry
        If oRegKey.GetValue(My.Application.Info.AssemblyName) Is Nothing Then
            ProgrammMitWindowsstartenToolStripMenuItem.Checked = False
        Else
            ProgrammMitWindowsstartenToolStripMenuItem.Checked = True
        End If

        oRegKey.Close()

    End Sub

    'start program when windows starts
    Private Sub ProgrammMitWindowsstartenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
        Handles ProgrammMitWindowsstartenToolStripMenuItem.Click

        'writing in registry only if admin
        If Not RegSecurity.IsAdmin() Then

            'if not admin then restart
            RegSecurity.RestartElevated()

        Else

            'registry key
            Dim oRegKey As Microsoft.Win32.RegistryKey

            'difference of 64 and 32 bit os
            If Environment.Is64BitOperatingSystem Then
                oRegKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Run", True)
            Else
                oRegKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("SOFTWARE\Microsoft\Windows\CurrentVersion\Run", True)
            End If

            'if autostart then delete entry and reversed
            If ProgrammMitWindowsstartenToolStripMenuItem.Checked Then
                oRegKey.DeleteValue(My.Application.Info.AssemblyName)
                ProgrammMitWindowsstartenToolStripMenuItem.Checked = True
            Else
                oRegKey.SetValue(My.Application.Info.AssemblyName, System.Windows.Forms.Application.ExecutablePath & " -a")
                ProgrammMitWindowsstartenToolStripMenuItem.Checked = True
            End If

            oRegKey.Close()

        End If

    End Sub

    'program loaded
    Private Sub frmBirthdayMessenger_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        'read birthday database
        Read_Data()

        'print birthdays today and if none then
        If Not Check_Birthday_Today() Then

            'if started by autostart then end
            Dim arguments As [String]() = Environment.GetCommandLineArgs()

            For n As Integer = arguments.GetLowerBound(0) To arguments.GetUpperBound(0)
                If arguments(n) = "-a" Then
                    End
                End If
            Next
        End If

        'set uac shield icon on autostart menuitem
        ProgrammMitWindowsstartenToolStripMenuItem.Image = GetUacShieldImage()

        'select today
        calView_DateChanged(Me, New DateRangeEventArgs(Today, Today))

        'select first
        lstNames.SelectedIndex = 0

    End Sub

    'read birthday database and save into birthday list
    Public Sub Read_Data()

        Dim content As String                                                       'whole file's content
        Dim name As String, preName As String, birthDay As Date, info As String     'temporary vars for 1 birthday list item
        Dim txtPath As String = Application.StartupPath & "\geburtstage.txt"        'path of textfile
        Dim pipePos As Integer                                                      'contains position of next pipe sign (also for next line signs)

        'skip if file does not exist
        If Not My.Computer.FileSystem.FileExists(txtPath) Then
            Return
        End If

        'read complete file
        content = My.Computer.FileSystem.ReadAllText(txtPath)

        'separate single information
        Do While content <> ""

            Try
                'date
                pipePos = content.IndexOf("|")                                      'look for separation sign
                birthDay = content.Remove(pipePos)                                  'get information
                content = content.Substring(pipePos + 1)                            'remove read information + separation sign

                'prename
                pipePos = content.IndexOf("|")
                preName = content.Remove(pipePos)
                content = content.Substring(pipePos + 1)

                'name
                pipePos = content.IndexOf("|")
                name = content.Remove(pipePos)
                content = content.Substring(pipePos + 1)

                'info
                pipePos = content.IndexOf(vbCrLf)
                info = content.Remove(pipePos)

                If pipePos < 0 Then                                                 'reached end of file?
                    content = ""
                Else
                    content = content.Substring(pipePos + vbCrLf.Length)
                End If

                'save temporary information in birthday list entry
                birthdays.Add(New BirthdayInfo(name, preName, birthDay, info))

                'mark in calendar
                calView.AddAnnuallyBoldedDate(birthDay)

            Catch ex As Exception
            End Try

        Loop

        'update calendar
        calView.UpdateBoldedDates()
    End Sub

    'write data into textfile
    Public Sub Write_Data()

        Dim content As String = ""                                                  'contains whole file
        Dim txtPath As String = Application.StartupPath & "\geburtstage.txt"        'path of textfile

        Sort_Birthdays()

        'print all birthday list items in content var
        For n As Integer = 0 To birthdays.Count - 1
            content = content & birthdays(n).birthDay & "|" & birthdays(n).preName & "|" & _
                birthdays(n).name & "|" & birthdays(n).info & vbCrLf
        Next n

        'write information into file
        Try
            My.Computer.FileSystem.WriteAllText(txtPath, content, False)
        Catch ex As Exception
            'maybe no permission to create and write file there
            MsgBox("Fehler beim Speichern der Geburtstage: " & vbCrLf & ex.Message, MsgBoxStyle.Critical, "Fehler")
        End Try

    End Sub

    'return order value of birthday entry
    'consider only month and day
    Public Function Get_Order_Value(ByVal birthDayIndex As Integer) As Double

        Return birthdays(birthDayIndex).birthDay.Month + _
            birthdays(birthDayIndex).birthDay.Day / 32

    End Function

    'actualize list and save to textfile
    Private Sub InDateispeichernToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InDateispeichernToolStripMenuItem.Click

        'save changes and write information into textfile
        Write_Data()

    End Sub

    'sort birthdays considering month and day values
    Private Sub Sort_Birthdays()    'out of msdn entry: listbox sort

        If (birthdays.Count > 1) Then

            Dim swapped As Boolean

            Do
                Dim counter As Integer = birthdays.Count - 1
                swapped = False
                While (counter > 0)

                    ' Compare the items
                    If Get_Order_Value(counter) < Get_Order_Value(counter - 1) Then

                        ' If true, swap the items.
                        Dim temp As Object = birthdays(counter)
                        birthdays(counter) = birthdays(counter - 1)
                        birthdays(counter - 1) = temp
                        swapped = True

                    End If
                    ' Decrement the counter.
                    counter -= 1
                End While
            Loop While (swapped = True)
        End If

    End Sub

    'exit program
    Private Sub frmBirthdayMessenger_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        InDateispeichernToolStripMenuItem.PerformClick()

    End Sub

    'delete selected entry
    Private Sub EintraglöschenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EintraglöschenToolStripMenuItem.Click

        Remove_Birthday(indexList(lstNames.SelectedIndex))

    End Sub

    'delete entry of this index
    Private Sub Remove_Birthday(ByVal birthDayIndex As Integer)

        Dim n As Integer = 0

        'only valid index
        If birthDayIndex > -1 And birthDayIndex < birthdays.Count Then
            'remove birthday
            birthdays.RemoveAt(birthDayIndex)

            'actualize lists
            calView.RemoveAnnuallyBoldedDate(calView.SelectionRange.Start)
            calView.UpdateBoldedDates()

            'delete nameitem
            If indexList(lstNames.SelectedIndex) = birthDayIndex Then
                lstNames.Items.RemoveAt(lstNames.SelectedIndex)
            End If

            'update indexlist
            Do While n < indexList.Count
                If indexList(n) > birthDayIndex Then
                    indexList(n) = indexList(n) - 1
                ElseIf indexList(n) = birthDayIndex Then
                    indexList.RemoveAt(n)
                    n = n - 1
                End If
                n = n + 1
            Loop

            lstNames.SelectedIndex = 0

        Else
            Beep()

        End If

    End Sub

    'are there birthdays today? return true if yes
    Private Function Check_Birthday_Today() As Boolean
        Dim result As Boolean = False

        'check all birthdays
        For n As Integer = 0 To birthdays.Count - 1
            If birthdays(n).birthDay.Month = Today.Date.Month And birthdays(n).birthDay.Day = Today.Date.Day Then
                'print information and mark as successfull
                MsgBox("Heute hat " & birthdays(n).preName & " " & birthdays(n).name & " Geburtstag.", _
                       MsgBoxStyle.Information, "Geburtstag heute")
                result = True
            End If
        Next n

        Return result
    End Function

    'print birthdays of last week
    Private Sub LetzteWocheToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LetzteWocheToolStripMenuItem.Click

        Dim messages As String = ""

        For n As Integer = 0 To birthdays.Count - 1
            'dates without years
            Dim todayDay As Date = New Date(1, Today.Month, Today.Day)
            Dim lastWeek As Date = New Date(1, Today.Month, Today.Day)
            lastWeek = lastWeek.AddDays(-7)
            Dim birthDay As Date = New Date(1, birthdays(n).birthDay.Month, birthdays(n).birthDay.Day)

            'if lastweek < compdate < today | consider not years
            If lastWeek <= birthDay And birthDay <= todayDay Then
                messages = messages & Print_Birthday(n) & vbCrLf
            End If
        Next

        If messages <> "" Then
            MsgBox("Geburtstage in der letzten Woche:" & vbCrLf & messages, MsgBoxStyle.Information, "Letzte Woche")
        Else
            MsgBox("Es gab keine Geburtstage in der letzten Woche.", MsgBoxStyle.Information, "Letzte Woche")
        End If

    End Sub

    'print birthdays of next 2 weeks
    Private Sub Nächste2WochenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Nächste2WochenToolStripMenuItem.Click

        Dim messages As String = ""

        For n As Integer = 0 To birthdays.Count - 1
            'dates without years
            Dim todayDay As Date = New Date(1, Today.Month, Today.Day)
            Dim next2Weeks As Date = New Date(1, Today.Month, Today.Day)
            next2Weeks = next2Weeks.AddDays(14)
            Dim birthDay As Date = New Date(1, birthdays(n).birthDay.Month, birthdays(n).birthDay.Day)

            'if lastweek < compdate < today | consider not years
            If next2Weeks >= birthDay And birthDay >= todayDay Then
                messages = messages & Print_Birthday(n) & vbCrLf
            End If
        Next

        If messages <> "" Then
            MsgBox("Geburtstage in den nächsten 2 Wochen:" & vbCrLf & messages, MsgBoxStyle.Information, "Nächste 2 Wochen")
        Else
            MsgBox("Es gibt keine Geburtstage in den nächsten 2 Wochen.", MsgBoxStyle.Information, "Nächste 2 Wochen")
        End If

    End Sub

    'print birthday of specified index
    Private Function Print_Birthday(ByVal index As Integer) As String

        Return birthdays(index).preName & " " & birthdays(index).name & " hat am " & birthdays(index).birthDay.Month & "." & _
            birthdays(index).birthDay.Day & ". Geburtstag."

    End Function

    'save current information into selected birthday list item
    Private Sub Save_Changes()

        'skip if no entry selected
        If lstNames.SelectedIndex = -1 Or ignoreChanges = True Then
            ignoreChanges = False
            Return
        End If

        'pipe signs must not be saved
        If txtPreName.Text.IndexOf("|") > -1 Then
            ignoreChanges = True
            txtPreName.Text = txtPreName.Text.Replace("|", "")
        End If
        If txtName.Text.IndexOf("|") > -1 Then
            ignoreChanges = True
            txtName.Text = txtName.Text.Replace("|", "")
        End If
        If rtbAdditional.Text.IndexOf("|") > -1 Then
            ignoreChanges = True
            rtbAdditional.Text = rtbAdditional.Text.Replace("|", "")
        End If

        'update exiting entry
        If indexList(lstNames.SelectedIndex) > -1 Then
            If birthdays(indexList(lstNames.SelectedIndex)).preName <> txtPreName.Text Or _
                birthdays(indexList(lstNames.SelectedIndex)).name <> txtName.Text Or _
                birthdays(indexList(lstNames.SelectedIndex)).info <> rtbAdditional.Text Then

                'delete if empty
                If txtPreName.Text = "" And _
                    txtName.Text = "" And _
                    rtbAdditional.Text = "" Then

                    Remove_Birthday(indexList(lstNames.SelectedIndex))

                Else    'else save changes
                    'save changes
                    birthdays(indexList(lstNames.SelectedIndex)) = New BirthdayInfo(txtName.Text, txtPreName.Text, _
                        birthdays(indexList(lstNames.SelectedIndex)).birthDay, rtbAdditional.Text)
                    'update list
                    lstNames.Items(lstNames.SelectedIndex) = birthdays(indexList(lstNames.SelectedIndex)).preName & _
                        " " & birthdays(indexList(lstNames.SelectedIndex)).name
                End If
            End If

            'create new entry did not exist, create new
        ElseIf indexList(lstNames.SelectedIndex) = -1 Then
            'if there is information to save
            If txtPreName.Text <> "" Or txtName.Text <> "" Or rtbAdditional.Text <> "" Then

                birthdays.Add(New BirthdayInfo(txtName.Text, txtPreName.Text, calView.SelectionRange.Start, rtbAdditional.Text))
                calView.AddAnnuallyBoldedDate(birthdays(birthdays.Count - 1).birthDay)

                'update lists
                indexList.Insert(indexList.Count - 1, birthdays.Count - 1)
                lstNames.Items.Insert(lstNames.Items.Count - 1, birthdays(birthdays.Count - 1).preName & " " & birthdays(birthdays.Count - 1).name)
                lstNames.SelectedIndex = lstNames.Items.Count - 2
                calView.UpdateBoldedDates()
            End If
        End If

    End Sub

    'show entries at selected date
    Private Sub calView_DateChanged(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DateRangeEventArgs) Handles calView.DateChanged

        'clear old
        indexList.Clear()
        lstNames.Items.Clear()

        'get all entries at this date
        For n As Integer = 0 To birthdays.Count - 1
            If birthdays(n).birthDay.Month = calView.SelectionRange.Start.Month And _
                birthdays(n).birthDay.Day = calView.SelectionRange.Start.Day Then
                indexList.Add(n)
                lstNames.Items.Add(birthdays(n).preName & " " & birthdays(n).name)
            End If
        Next

        'add new entry
        indexList.Add(-1)
        lstNames.Items.Add("Neuer Eintrag")

        'select first
        lstNames.SelectedIndex = 0

        'update oldDate
        oldDate = calView.SelectionRange.Start

    End Sub

    'show information of selected birthday
    Private Sub lstNames_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstNames.SelectedIndexChanged

        'Try
        If lstNames.SelectedIndex = -1 Then
            Return
        End If

        'show selected birthday's information
        If indexList(lstNames.SelectedIndex) > -1 Then
            If txtName.Text <> birthdays(indexList(lstNames.SelectedIndex)).name Then
                ignoreChanges = True
                txtName.Text = birthdays(indexList(lstNames.SelectedIndex)).name
            End If
            If txtPreName.Text <> birthdays(indexList(lstNames.SelectedIndex)).preName Then
                ignoreChanges = True
                txtPreName.Text = birthdays(indexList(lstNames.SelectedIndex)).preName
            End If
            If rtbAdditional.Text <> birthdays(indexList(lstNames.SelectedIndex)).info Then
                ignoreChanges = True
                rtbAdditional.Text = birthdays(indexList(lstNames.SelectedIndex)).info
            End If
            'or show empty entry
        Else
            If txtName.Text <> "" Then
                ignoreChanges = True
                txtName.Text = ""
            End If
            If txtPreName.Text <> "" Then
                ignoreChanges = True
                txtPreName.Text = ""
            End If
            If rtbAdditional.Text <> "" Then
                ignoreChanges = True
                rtbAdditional.Text = ""
            End If
            txtPreName.Focus()
        End If

        'Catch ex As Exception
        '    Return
        'End Try

    End Sub

    Private Sub txtName_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtName.TextChanged
        Save_Changes()
    End Sub

    Private Sub txtPreName_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtPreName.TextChanged
        Save_Changes()
    End Sub

    Private Sub rtbAdditional_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rtbAdditional.TextChanged
        Save_Changes()
    End Sub
End Class
